@@ -20,11 +20,22 @@ package Finance::Quote::AlphaVantage;
 
 require 5.005;
 
-our $VERSION = '1.46'; # VERSION
+our $VERSION = '1.48'; # VERSION
 
 use strict;
 use JSON qw( decode_json );
 use HTTP::Request::Common;
+use Time::HiRes qw(usleep clock_gettime);
+
+# Alpha Vantage recommends that API call frequency does not extend far
+# beyond ~1 call per second so that they can continue to deliver
+# optimal server-side performance:
+#   https://www.alphavantage.co/support/#api-key
+our @alphaqueries=();
+my $maxQueries = { quantity =>5 , seconds => 60}; # no more than x
+                                                  # queries per y
+                                                  # seconds, based on
+                                                  # https://www.alphavantage.co/support/#support
 
 my $ALPHAVANTAGE_URL =
     'https://www.alphavantage.co/query?function=TIME_SERIES_DAILY&outputsize=compact&datatype=json';
@@ -60,6 +71,7 @@ my %currencies_by_suffix = (
     '.H'   => "EUR",    # 		Hamburg
     '.HA'  => "EUR",    # 		Hanover
     '.MU'  => "EUR",    # 		Munich
+    '.ME'  => "RUB",    # Russia	Moscow
     '.SG'  => "EUR",    # 		Stuttgart
     '.DE'  => "EUR",    # 		XETRA
     '.HK'  => "HKD",    # Hong Kong
@@ -87,6 +99,8 @@ my %currencies_by_suffix = (
     '.MA'  => "EUR",    # 		Madrid
     '.VA'  => "EUR",    # 		Valence
     '.ST'  => "SEK",    # Sweden		Stockholm
+    '.STO' => "SEK",    # Sweden		Stockholm
+    '.HE'  => "EUR",    # Finland		Helsinki
     '.S'   => "CHF",    # Switzerland	Zurich
     '.TW'  => "TWD",    # Taiwan		Taiwan Stock Exchange
     '.TWO' => "TWD",    # 		OTC
@@ -94,26 +108,55 @@ my %currencies_by_suffix = (
     '.TH'  => "THB",    # 		??? From Asia.pm, (in Thai Baht)
     '.L'   => "GBP",    # United Kingdom	London
     '.IL'  => "USD",    # United Kingdom	London USD*100
+    '.VX'  => "CHF",    # Switzerland
+    '.SW'  => "CHF",    # Switzerland
 );
 
 
 sub methods {
-    return ( alphavantage => \&alphavantage, );
+    return ( alphavantage => \&alphavantage,
+             canada       => \&alphavantage,
+             usa          => \&alphavantage,
+             nyse         => \&alphavantage,
+             nasdaq       => \&alphavantage,
+             vanguard     => \&alphavantage,
+    );
+}
 
-    our @labels = qw/date isodate open high low close volume last/;
+{
+    my @labels = qw/date isodate open high low close volume last/;
 
     sub labels {
         return ( alphavantage => \@labels, );
     }
 }
 
+sub sleep_before_query {
+    # wait till we can query again
+    my $q = $maxQueries->{quantity}-1;
+    if ( $#alphaqueries >= $q ) {
+        my $time_since_x_queries = clock_gettime()-$alphaqueries[$q];
+        # print STDERR "LAST QUERY $time_since_x_queries\n";
+        if ($time_since_x_queries < $maxQueries->{seconds}) {
+            my $sleeptime = ($maxQueries->{seconds} - $time_since_x_queries) * 1000000;
+            # print STDERR "SLEEP $sleeptime\n";
+            usleep( $sleeptime );
+            # print STDERR "CONTINUE\n";
+        }
+    }
+    unshift @alphaqueries, clock_gettime();
+    pop @alphaqueries while $#alphaqueries>$q; # remove unnecessary data
+    # print STDERR join(",",@alphaqueries)."\n";
+}
+
 sub alphavantage {
     my $quoter = shift;
 
     my @stocks = @_;
     my $quantity = @stocks;
-    my ( %info, $reply, $url );
+    my ( %info, $reply, $url, $code, $desc, $body );
     my $ua = $quoter->user_agent();
+    my $launch_time = clock_gettime();
 
     foreach my $stock (@stocks) {
 
@@ -130,11 +173,20 @@ sub alphavantage {
             . $ALPHAVANTAGE_API_KEY
             . '&symbol='
             . $stock;
-        $reply = $ua->request( GET $url);
 
-        my $code = $reply->code;
-        my $desc = HTTP::Status::status_message($code);
-        my $body = $reply->content;
+        my $get_content = sub {
+            sleep_before_query();
+            my $time=int(clock_gettime()-$launch_time);
+            # print STDERR "Query at:".$time."\n";
+            $reply = $ua->request( GET $url);
+
+            $code = $reply->code;
+            $desc = HTTP::Status::status_message($code);
+            $body = $reply->content;
+        };
+
+        &$get_content();
+
         if ($code != 200) {
             $info{ $stock, 'success' } = 0;
             $info{ $stock, 'errormsg' } = $desc;
@@ -148,6 +200,16 @@ sub alphavantage {
             $info{ $stock, 'errormsg' } = $@;
         }
 
+        my $try_cnt = 0;
+        while (($try_cnt < 5) && ($json_data->{'Note'})) {
+            # print STDERR "INFORMATION:".$json_data->{'Note'}."\n";
+            # print STDERR "ADDITIONAL SLEEPING HERE !";
+            sleep (20);
+            &$get_content();
+            eval {$json_data = JSON::decode_json $body};
+            $try_cnt += 1;
+        }
+
         if ( !$json_data || $json_data->{'Error Message'} ) {
             $info{ $stock, 'success' } = 0;
             $info{ $stock, 'errormsg' } =
@@ -155,6 +217,12 @@ sub alphavantage {
             next;
         }
 
+        if (!$json_data->{'Meta Data'}) {
+            $info{ $stock, 'success' } = 0;
+            $info{ $stock, 'errormsg' } = ( $json_data->{'Information'} || "No useable data returned" ) ;
+            next;
+        }
+
         my $last_refresh = $json_data->{'Meta Data'}->{'3. Last Refreshed'}; # when market is open this returns an isodate + time, otherwise only the isodate
         $last_refresh = substr($last_refresh,0,10);  # remove time if returned
         if ( !$last_refresh ) {
@@ -203,7 +271,7 @@ sub alphavantage {
 
         # deduce currency
         if ( $stock =~ /(\..*)/ ) {
-            my $suffix = $1;
+            my $suffix = uc $1;
             if ( $currencies_by_suffix{$suffix} ) {
                 $info{ $stock, 'currency' } = $currencies_by_suffix{$suffix};
 
@@ -233,8 +301,6 @@ sub alphavantage {
 
         $info{ $stock, "currency_set_by_fq" } = 1;
 
-        $quantity--;
-        select(undef, undef, undef, .7) if ($quantity);
     }
 
     return wantarray() ? %info : \%info;
