@@ -1,5 +1,4 @@
 #!/usr/bin/perl -w
-# vi: set ts=4 sw=4 noai ic showmode showmatch: 
 
 #  MorningstarCH.pm
 #
@@ -8,9 +7,9 @@
 #
 #  author: Manuel Friedli (manuel@fritteli.ch)
 #
-#  version: 0.2 Updated version - 30 July 2025
+#  version: 0.1 Initial version - 02 March 2019
 #
-#  This file is heavily based on MorningstarUK.pm by Martin Sadler
+#  This file is heavily based on MStaruk.pm by Martin Sadler
 #  (martinsadler@users.sourceforge.net), version 0.1, 01 April 2013
 #
 #    This program is free software; you can redistribute it and/or modify
@@ -25,12 +24,13 @@
 #
 #    You should have received a copy of the GNU General Public License
 #    along with this program; if not, write to the Free Software
-#    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
-#    02110-1301, USA
+#    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
+#    02111-1307, USA
 #
 
 
 package Finance::Quote::MorningstarCH;
+require 5.005;
 
 use strict;
 use warnings;
@@ -38,51 +38,31 @@ use warnings;
 # URLs
 use vars qw($VERSION $MSTARCH_NEXT_URL $MSTARCH_LOOK_UP $MSTARCH_MAIN_URL);
 
-use constant DEBUG => $ENV{DEBUG};
-use if DEBUG, 'Smart::Comments';
-
 use LWP::Simple;
 use LWP::UserAgent;
 use HTTP::Request::Common;
 use HTTP::Cookies;
-use JSON qw(decode_json);
-use Text::Template;
 
-our $VERSION = '1.68_01'; # TRIAL VERSION
+our $VERSION = '1.48'; # VERSION
 
-$MSTARCH_MAIN_URL   =   "https://www.morningstar.ch";
-$MSTARCH_LOOK_UP    =   "https://www.morningstar.ch/ch/funds/SecuritySearchResults.aspx?search=";
-$MSTARCH_NEXT_URL = Text::Template->new( TYPE => 'STRING', SOURCE => 'https://api-global.morningstar.com/sal-service/v1/fund/quote/v7/{$secid}/data?fundServCode=&showAnalystRatingChinaFund=false&showAnalystRating=false&hideesg=false&region=EEA&languageId=en-eu&locale=en-eu&clientId=MDC&benchmarkId=mstarorcat&component=sal-mip-investment-overview&version=4.65.0' );
+$MSTARCH_MAIN_URL   =	"http://www.morningstar.ch";
+$MSTARCH_LOOK_UP    =	"http://www.morningstar.ch/ch/funds/SecuritySearchResults.aspx?search=";
+$MSTARCH_NEXT_URL   =	"http://www.morningstar.ch/ch/funds/snapshot/snapshot.aspx?id=";
 
-# FIXME - Needs cleanup
+# FIXME -
 
-our $DISPLAY    = 'Morningstar CH';
-our @LABELS = qw/name currency last date price nav source iso_date method success errormsg/;
-our $METHODHASH = {subroutine => \&mstarch_fund,
-                   display => $DISPLAY,
-                   labels => \@LABELS};
+sub methods { return (morningstarch => \&morningstarch_fund); }
 
-sub methodinfo {
-    return (
-        morningstarch => $METHODHASH,
-        mstarch       => $METHODHASH,
-    );
-}
+{
+    my @labels = qw/name currency last date time price nav source iso_date method net p_change success errormsg/;
 
-sub methods {
-	my %m = methodinfo();
-	return map {$_ => $m{$_}{subroutine} } keys %m;
-}
-
-sub labels {
-	my %m = methodinfo();
-	return map {$_ => [@{$m{$_}{labels}}] } keys %m;
+    sub labels { return (morningstarch => \@labels); }
 }
 
 #
 # =======================================================================
 
-sub mstarch_fund  {
+sub morningstarch_fund  {
     my $quoter = shift;
     my @symbols = @_;
 
@@ -126,22 +106,18 @@ sub mstarch_fund  {
 		        "Error - failed to retrieve fund data";
 		    next;
 	    }
-
-		### [<now>] webdoc: $webdoc
-
 	    $fundquote {$code, "symbol"} = $code;
 	    $fundquote {$code, "source"} = $MSTARCH_MAIN_URL;
 
 # Find name by regexp
 
-        my ($name, $nexturl, $secid);
+        my ($name, $nexturl, $isin);
  		if ($webdoc =~
-        m[<td class="msDataText searchLink"><a href="(.*?id=([A-Z0-9]+))">(.*?)</a></td><td class="msDataText searchIsin"><span>[a-zA-Z]{2}[a-zA-Z0-9]{9}\d(.*)</span></td>] )
+        m[<td class="msDataText searchLink"><a href="(.*?)">(.*?)</a></td><td class="msDataText searchIsin"><span>[a-zA-Z]{2}[a-zA-Z0-9]{9}\d(.*)</span></td>] )
         {
             $nexturl = $1;
-            $secid = $2;
-			### [<now>] secID: $secid
-            $name = $3;
+            $name = $2;
+            $isin = $3;
         }
 
 		if (!defined($name)) {
@@ -162,42 +138,46 @@ sub mstarch_fund  {
 		    next;
 		}
 
-		$nexturl = $MSTARCH_NEXT_URL->fill_in(HASH => {secid => $secid});
-		$ua->default_header('Apikey' => 'lstzFDEOhfFNMLikKa0am9mgEKLBl49T');
+# modify $nexturl to remove html escape encoding for the Ampersand (&) character
 
-		### [<now>] NextURL: $nexturl
+		$nexturl =~ s/&amp;/&/;
 
-		my $response = $ua->request( GET $nexturl );
+# Now need to look-up next page using $next_url
 
-		if ($response->code != 200 ) {
-			$fundquote {$code,"success"} = 0;
-			$fundquote {$code,"errormsg"} = "Error - $code not found";
-			next;
-		}
-
-		$webdoc = $response->content;
-		### [<now>] 2nd Webdoc: $webdoc
-
-		my $json;
-		eval {$json = decode_json $webdoc};
-        if ($@) {
+        $webdoc  = get($MSTARCH_MAIN_URL.$nexturl);
+        if (!$webdoc)
+        {
 	        # serious error, report it and give up
 		    $fundquote {$code,"success"} = 0;
 		    $fundquote {$code,"errormsg"} =
-		        "Error - failed to retrieve fund data - could not decode JSON";
+		        "Error - failed to retrieve fund data";
 		    next;
 	    }
-		### [<now>] JSON: $json
 
 # Find date, currency and price all in one table row
 
-		my ($date, $time);
-		my $currency = $json->{'currency'};
-		my $price    = $json->{'nav'};
-		if ( $json->{'latestPriceDate'} =~ m|(\d{4}-\d{2}-\d{2})T(\d{2}:\d{2}):| ) {
-			$date = $1;
-			$time = $2;
+		my ($currency, $date, $price, $pchange);
+		if ($webdoc =~
+		m[<td class="line heading">NAV<span class="heading"><br />([0-9]{2}\.[0-9]{2}\.[0-9]{4})</span>.*([A-Z]{3}).([0-9\,\.]+).*>([0-9\,\.\-]+)%?] )
+        {
+
+            $date = $1;
+            $currency = $2;
+            $price = $3;
+            $pchange = $4;
+        }
+
+		if (!defined($pchange)) {
+			# not a serious error - don't report it ....
+#			$fundquote {$code,"success"} = 0;
+			# ... but set a useful message ....
+			$fundquote {$code,"errormsg"} = "Warning - failed to find net or %-age change";
+			# set to (minus)zero
+            $pchange = -0.00;
+			# ... and continue
 		}
+		$pchange =~ s/,/./;
+		$fundquote {$code, "p_change"} = $pchange;	# set %-change
 
 		if (!defined($date)) {
 			# not a serious error - don't report it ....
@@ -210,7 +190,7 @@ sub mstarch_fund  {
 		}
 		else
 		{
-		    $quoter->store_date(\%fundquote, $code, {isodate => $date});
+		    $quoter->store_date(\%fundquote, $code, {eurodate => $date});
 		}
 
 		if (!defined($price)) {
@@ -219,6 +199,7 @@ sub mstarch_fund  {
 			$fundquote {$code,"errormsg"} = "Error - failed to find a price";
 			next;
 		}
+		$price =~ s/,/./;
 
 		if (!defined($currency)) {
 	    	# serious error, report it and give up
@@ -229,22 +210,26 @@ sub mstarch_fund  {
 
 		# defer setting currency and price until we've dealt with possible GBX currency...
 
-# deal with GBX pricing of UK unit trusts
+# Calculate net change - it's not included in the morningstar factsheets
 
-		if ($currency eq "GBX")
-		{
-			$currency = "GBP" ;
-			$price = $price / 100 ;
-		}
+		my $net = ($price * $pchange) / 100 ;
 
 		# now set prices and currency
 
 		$fundquote {$code, "price"} = $price;
 		$fundquote {$code, "last"} = $price;
 		$fundquote {$code, "nav"} = $price;
+		$fundquote {$code, "net"} = $net;
 		$fundquote {$code, "currency"} = $currency;
 
-		$fundquote {$code, "method"} = "morningstaruk";   # set method
+# Set a dummy time as gnucash insists on having a valid format
+
+		my $time = "12:00";     # set to Midday if no time supplied ???
+                                # gnucash insists on having a valid-format
+
+		$fundquote {$code, "time"} = $time; # set time
+
+		$fundquote {$code, "method"} = "morningstarch";   # set method
 
 	}
 
@@ -255,27 +240,27 @@ sub mstarch_fund  {
 
 =head1 NAME
 
-Finance::Quote::MorningstarCH - Obtain CH Unit Trust quotes from morningstar.ch.
+Finance::Quote::morningstarch - Obtain CH Unit Trust quotes from morningstar.ch.
 
 =head1 SYNOPSIS
 
     $q = Finance::Quote->new;
 
-    %info = $q->fetch("morningstarch","<isin> ...");  # Only query morningstar.ch using ISINs
+    %info = Finance::Quote->fetch("morningstarch","<isin> ...");  # Only query morningstar.ch using ISINs
 
 =head1 DESCRIPTION
 
 This module fetches information from the MorningStar Funds service,
-https://morningstar.com/ch/.
+http://morningstar.com/ch/.
 
 Funds are identified by their ISIN code.
 
 This module is loaded by default on a Finance::Quote object. It's
-also possible to load it explicitly by placing "morningstarch" in the argument
+also possible to load it explicity by placing "morningstarch" in the argument
 list to Finance::Quote->new().
 
 Information obtained by this module may be covered by Morningstar
-terms and conditions See https://morningstar.ch/ for details.
+terms and conditions See http://morningstar.ch/ for details.
 
 =head2 Stocks And Indices
 
@@ -284,7 +269,7 @@ Trusts and OEICs prices and other information from morningstar.ch.
 
 =head1 LABELS RETURNED
 
-The following labels may be returned by Finance::Quote::MorningstarCH :
+The following labels may be returned by Finance::Quote::morningstarch :
 
     name, currency, last, date, time, price, nav, source, method,
     iso_date, net, p_change, success, errormsg.
@@ -292,17 +277,17 @@ The following labels may be returned by Finance::Quote::MorningstarCH :
 
 =head1 SEE ALSO
 
-Morning Star websites, https://morningstar.ch
+Morning Star websites, http://morningstar.ch
 
 
 =head1 AUTHOR
 
 Manuel Friedli, E<lt>manuel@fritteli.chE<gt>
-Based heavily on the work of Martin Sadler, E<lt>martinsadler@users.sourceforge.netE<gt>, many thanks!
+Based heavily on the work of Martin Sadler E<lt>martinsadler@users.sourceforge.netE<gt>, many thanks!
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (C) 2019, 2025 by Manuel Friedli
+Copyright (C) 2019 by Manuel Friedli
 
 This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself, either Perl version 5.10.1 or,
